@@ -1,4 +1,4 @@
-# Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,7 @@ from table_templates import TableTemplateManager
 
 from sqlide_tableman_ext import CreateIndexForm
 from sqlide_catalogman_ext import show_schema_manager
+from functools import reduce
 
 def esc_ident(s):
     # escape SQL identifier and add backtics only if needed
@@ -121,7 +122,7 @@ def handleLiveTreeContextMenu(name, sender, args):
                 selection_type = s.owner.type+':'+s.type
             column_selected = True
         else:
-            print "Unhandled type", s.type
+            print("Unhandled type", s.type)
 
     if mixed:
         selection_type = None
@@ -136,15 +137,18 @@ def handleLiveTreeContextMenu(name, sender, args):
 
         if False and object_selected and selection_type == 'db.Table':
             item = mforms.newMenuItem(_("Export Data to CSV File..."))
-            item.set_name("export_table_csv")
+            item.set_name("Export Table to CSV")
+            item.setInternalName("export_table_csv")
             menu.insert_item(index-1, item)
             index += 1
             item = mforms.newMenuItem(_("Import Data from CSV File..."))
-            item.set_name("import_table_csv")
+            item.set_name("Import Table to CSV")
+            item.setInternalName("import_table_csv")
             menu.insert_item(index-1, item)
             index += 1
             item = mforms.newMenuItem(_("Dump Data to SQL File..."))
-            item.set_name("dump_table")
+            item.set_name("Dump Table")
+            item.setInternalName("dump_table")
             menu.insert_item(index-1, item)
             index += 1
 
@@ -152,40 +156,42 @@ def handleLiveTreeContextMenu(name, sender, args):
             # Add menu items for generating code
             code_items = [
                           # Caption, callback name, supported node types, enable condition, mixed node types allowed
-                          (_("Name"), 'name_short', ['db.Schema'], len(selection) > 0, False),
-                          (_("Name (short)"), 'name_short', ['tables', 'views', 'functions', 'storedProcedures', 'db.Table', 'db.Table:db.Column', 'db.View:db.Column', 'db.View', 'db.StoredProcedure', 'db.Function'], len(selection) > 0, True),
-                          (_("Name (long)"), 'name_long', ['tables', 'views', 'functions', 'storedProcedures', 'db.Table', 'db.Table:db.Column', 'db.View:db.Column', 'db.View', 'db.StoredProcedure', 'db.Function'], len(selection) > 0, True),
-                          (_("Select All Statement"), 'select_all_statement', ['db.Table', 'db.View'], len(selection) > 0, False),
-                          (_("Select All Statement"), 'select_all_statement', ['columns'], len(selection) == 1 and selection[0].type == 'columns', False),
-                          (_("Select Columns Statement"), 'select_columns_statement', ['db.Table:db.Column', 'db.View:db.Column'], len(unique_tables) == 1, False),
-                          (_("Insert Statement"), 'insert_all_statement', ['db.Table', 'columns'], len(selection) > 0, False),
-                          (_("Insert Statement"), 'insert_columns_statement', ['db.Table:db.Column'], len(selection) > 0, False),
-                          (_("Update Statement"), 'update_all_statement', ['db.Table', 'columns'], len(selection) > 0, False),
-                          (_("Update Statement"), 'update_columns_statement', ['db.Table:db.Column'], len(selection) > 0, False),
-                          (_("Delete Statement"), 'delete_statement', ['db.Table'], len(selection) > 0, False),
-                          (_("Create Statement"), 'create_statement', ['db.Schema', 'db.Table', 'db.View', 'db.StoredProcedure', 'db.Function'], len(selection) > 0, False),
-                          (_("Procedure Call"), 'call_procedure', ['db.StoredProcedure'], len(selection) > 0, False),
-                          (_("Function Call"), 'call_function', ['db.Function'], len(selection) > 0, False),
-                          (None, None, ['db.Table'], None, False),
-                          (_("Join Selected Tables"), 'build_joined_select', ['db.Table'], len(selection) == 2, False),
-                          (_("Delete with References"), 'build_cascaded_delete', ['db.Table'], len(selection) == 1, False),
-                          (_("Select Row References"), 'build_cascaded_select', ['db.Table'], len(selection) == 1, False),
+                          (_("Name"), "Name", 'name_short', ['db.Schema'], len(selection) > 0, False),
+                          (_("Name (short)"), "Short Name", 'name_short', ['tables', 'views', 'functions', 'storedProcedures', 'db.Table', 'db.Table:db.Column', 'db.View:db.Column', 'db.View', 'db.StoredProcedure', 'db.Function'], len(selection) > 0, True),
+                          (_("Name (long)"), "Long Name", 'name_long', ['tables', 'views', 'functions', 'storedProcedures', 'db.Table', 'db.Table:db.Column', 'db.View:db.Column', 'db.View', 'db.StoredProcedure', 'db.Function'], len(selection) > 0, True),
+                          (_("Select All Statement"), "Select All Statement", 'select_all_statement', ['db.Table', 'db.View'], len(selection) > 0, False),
+                          (_("Select All Statement"), "Select All Statement", 'select_all_statement', ['columns'], len(selection) == 1 and selection[0].type == 'columns', False),
+                          (_("Select Columns Statement"), "Select Columns Statement", 'select_columns_statement', ['db.Table:db.Column', 'db.View:db.Column'], len(unique_tables) == 1, False),
+                          (_("Insert Statement"), "Insert Statement", 'insert_all_statement', ['db.Table', 'columns'], len(selection) > 0, False),
+                          (_("Insert Statement"), "Insert Statement", 'insert_columns_statement', ['db.Table:db.Column'], len(selection) > 0, False),
+                          (_("Update Statement"), "Update Statement", 'update_all_statement', ['db.Table', 'columns'], len(selection) > 0, False),
+                          (_("Update Statement"), "Update Statement", 'update_columns_statement', ['db.Table:db.Column'], len(selection) > 0, False),
+                          (_("Delete Statement"), "Delete Statement", 'delete_statement', ['db.Table'], len(selection) > 0, False),
+                          (_("Create Statement"), "Create Statement", 'create_statement', ['db.Schema', 'db.Table', 'db.View', 'db.StoredProcedure', 'db.Function'], len(selection) > 0, False),
+                          (_("Procedure Call"), "Procedure Call", 'call_procedure', ['db.StoredProcedure'], len(selection) > 0, False),
+                          (_("Function Call"), "Function Call", 'call_function', ['db.Function'], len(selection) > 0, False),
+                          (None, None, None, ['db.Table'], None, False),
+                          (_("Join Selected Tables"), "Join Selected Tables", 'build_joined_select', ['db.Table'], len(selection) == 2, False),
+                          (_("Delete with References"), "Delete With References", 'build_cascaded_delete', ['db.Table'], len(selection) == 1, False),
+                          (_("Select Row References"), "Select Row References", 'build_cascaded_select', ['db.Table'], len(selection) == 1, False),
                           ]
 
             copy_submenu = mforms.newMenuItem("Copy to Clipboard")
-            copy_submenu.set_name("copy_to_clipboard")
+            copy_submenu.set_name("Copy to Clipboard")
+            copy_submenu.setInternalName("copy_to_clipboard")
             menu.insert_item(index, copy_submenu)
             index += 1
 
             send_submenu = mforms.newMenuItem("Send to SQL Editor")
-            send_submenu.set_name("send_to_editor")
+            send_submenu.set_name("Send to Editor")
+            send_submenu.setInternalName("send_to_editor")
             menu.insert_item(index, send_submenu)
             index += 1
 
             gencopy = CodeGenerator(sender, selection, False)
             gensend = CodeGenerator(sender, selection, True)
 
-            for caption, name, types, enabled, allow_mixed in code_items:
+            for caption, accessibilityName, name, types, enabled, allow_mixed in code_items:
                 if not allow_mixed and mixed:
                     continue
                 if types and selection_type not in types:
@@ -194,10 +200,10 @@ def handleLiveTreeContextMenu(name, sender, args):
                     copy_submenu.add_separator()
                     send_submenu.add_separator()
                 else:
-                    item = copy_submenu.add_item_with_title(caption, getattr(gencopy, name), name)
+                    item = copy_submenu.add_item_with_title(caption, getattr(gencopy, name), accessibilityName, "")
                     item.set_enabled(bool(enabled))
 
-                    item = send_submenu.add_item_with_title(caption, getattr(gensend, name), name)
+                    item = send_submenu.add_item_with_title(caption, getattr(gensend, name), accessibilityName, "")
                     item.set_enabled(bool(enabled))
             needs_separator = True
         else:
@@ -222,9 +228,9 @@ def handleLiveTreeContextMenu(name, sender, args):
                 #        if selection[0].type == 'tables':
                 #            it.set_enabled(False)
                 for templ in template_manager.templates:
-                    item.add_item_with_title(templ.name, lambda templ=templ, schema=selection[0].schemaName: template_manager.create_table_like_template(sender, schema, templ))
+                    item.add_item_with_title(templ.name, lambda templ=templ, schema=selection[0].schemaName: template_manager.create_table_like_template(sender, schema, templ), templ.name, "")
                 item.add_separator()
-                item.add_item_with_title("Edit Templates...", template_manager.edit_templates)
+                item.add_item_with_title("Edit Templates...", template_manager.edit_templates, "Edit Templates", "")
                 
         if selection_type is None:
             if needs_separator:
@@ -368,7 +374,7 @@ def do_truncate_table(editor, selection):
             stmt = "TRUNCATE %s" % table
             editor.executeManagementCommand(stmt, 1)
             count += 1
-        except Exception, exc:
+        except Exception as exc:
             if count < len(tables)-1:
                 if mforms.Utilities.show_error("Could not Truncate Table", str(exc)+"\nClick Cancel to stop truncating other tables.\n\n"+stmt, "OK", "Cancel", "") == mforms.ResultCancel:
                     break
@@ -427,7 +433,7 @@ def do_drop_object(editor, selection):
             stmt = "DROP INDEX %s ON %s.%s" % (esc_ident(obj.name), esc_ident(obj.owner.schemaName), esc_ident(obj.owner.name))
             object_types.add("Index")
         else:
-            print "Unsupported type for drop", obj.type
+            print("Unsupported type for drop", obj.type)
         statements.append((1, stmt))
 
     # multiple tables can be dropped in the same command (probably so that we dont get ref constraint errors)
@@ -486,7 +492,7 @@ def do_drop_object(editor, selection):
             #This should be run in background because of the bug: #71327
             editor.executeCommand(stmt,1 ,1)
             drop_count += c
-        except Exception, exc:
+        except Exception as exc:
             if i < len(statements)-1:
                 if mforms.Utilities.show_error("Could not Drop Object", str(exc)+"\nClick Cancel to stop dropping other objects.\n\n"+stmt, "OK", "Cancel", "") == mforms.ResultCancel:
                     break
@@ -576,7 +582,7 @@ class DependencyAnalyzer:
         distance = {}
 
         # initialization
-        for table in self.tables_by_name.values():
+        for table in list(self.tables_by_name.values()):
             predecessor[table] = None
             distance[table] = 99999999
         distance[from_table] = 0
@@ -594,7 +600,7 @@ class DependencyAnalyzer:
             return v
 
         # main loop
-        remaining = self.tables_by_name.values()
+        remaining = list(self.tables_by_name.values())
         while remaining:
             u = get_cheapest(remaining)
             for v in self.get_referenced_tables(u):
@@ -651,7 +657,7 @@ def join_tables(info, tables, auto_add_missing):
                 if auto_add_missing:
                     fks = info.find_foreign_key_path_between(t1, t2) or info.find_foreign_key_path_between(t2, t1)
                     if not fks:
-                        print "Could not find path from %s to %s" % (t1.name, t2.name)
+                        print("Could not find path from %s to %s" % (t1.name, t2.name))
                         return None
                     joins.update(fks)
                 else:
@@ -694,7 +700,7 @@ class CodeGenerator:
 
     def get_table_columns(self, schema, table):
         info_key = "%s:%s.%s" % (self.editor.__id__, schema, table)
-        if table_column_cache.has_key(info_key):
+        if info_key in table_column_cache:
             return table_column_cache[info_key]
 
         rs = self.editor.executeManagementQuery("SHOW COLUMNS FROM %s.%s" % (esc_ident(schema), esc_ident(table)), 0)
@@ -708,7 +714,7 @@ class CodeGenerator:
                 columns.append((name, is_key, default))
                 ok = rs.nextRow()
             table_column_cache[info_key] = columns
-            print columns
+            print(columns)
             return columns
         else:
             return None
@@ -835,16 +841,16 @@ class CodeGenerator:
                 rs = self.editor.executeManagementQuery("SHOW CREATE VIEW %s.%s" % (esc_ident(obj.schemaName), esc_ident(obj.name)), 0)
                 field_name = 'Create View'
             else:
-                print "Unsupported type", obj.type
+                print("Unsupported type", obj.type)
                 continue
             if rs and rs.goToFirstRow():
                 sql = rs.stringFieldValueByName(field_name)
                 if not sql:
-                    print "No field %s for %s" % (field_name, obj.name)
+                    print("No field %s for %s" % (field_name, obj.name))
                 else:
                     parts.append(wrapper % sql)
             else:
-                print "Couldn't fetch create code for %s" % obj.name
+                print("Couldn't fetch create code for %s" % obj.name)
 
         self.send("\n".join(parts))
 
