@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2020, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0,
@@ -75,8 +75,7 @@ def backupConnections():
     file_chooser.set_extensions('ZIP Files (*.zip)|*.zip', 'import')
     if file_chooser.run_modal() == mforms.ResultOk:
         backup_path = file_chooser.get_path()
-        if isinstance(backup_path, unicode):
-            backup_path = backup_path.encode('utf-8')
+
         try:
             backup_file = zipfile.ZipFile(backup_path, 'w', zipfile.ZIP_DEFLATED)
         except Exception:
@@ -122,12 +121,12 @@ def restoreConnections():
                 connections_file = tempfile.NamedTemporaryFile(delete=False)
                 connections_file.write(backup_file.read('connections.xml'))
                 connections_file.close()
-            except KeyError, error:
+            except KeyError as error:
                 mforms.Utilities.show_error('Restore Connections Error', 'The selected file is not a valid backup file '
-                                            'or the file is corrupted: %s.' % error.message,
+                                            'or the file is corrupted: %s.' % str(error),
                                             'OK', '', '')
                 grt.log_error('restoreConnections', 'The selected file is not a valid backup file '
-                              'or the file is corrupted: %s.' % error.message)
+                              'or the file is corrupted: %s.' % str(error))
                 return
 
             connections = grt.unserialize(connections_file.name)
@@ -198,12 +197,12 @@ def restoreConnections():
                 message.append(', which were not restored.')
                 mforms.Utilities.show_warning('Restore Connections', ''.join(message), 'OK', '', '')
             
-        except zipfile.BadZipfile, error:
+        except zipfile.BadZipfile as error:
             mforms.Utilities.show_error('Restore Connections Error', 'The selected file is not a valid backup file '
                                         'or the file is corrupted.',
                                         'OK', '', '')
             grt.log_error('restoreConnections', 'The selected file is not a valid backup file or the file is corrupted: %s\n' % error)
-        except IOError, error:
+        except IOError as error:
             mforms.Utilities.show_error('Restore Connections Error', 'Cannot read from file. Please check this file '
                                         'permissions and try again.',
                                         'OK', '', '')
@@ -302,7 +301,7 @@ def connectionFromString(connstr):
             conn.parameterValues["socket"] = socket
 
         hostIdentifier = conn.driver.hostIdentifierTemplate
-        for key, value in conn.parameterValues.items():
+        for key, value in list(conn.parameterValues.items()):
             hostIdentifier = hostIdentifier.replace("%"+key+"%", str(value))
         conn.hostIdentifier = hostIdentifier
 
@@ -321,7 +320,7 @@ def copyConnectionString(conn):
 @ModuleInfo.plugin("wb.tools.copyJDBCConnectionString", caption="Copy JDBC Connection String to Clipboard", input= [wbinputs.selectedConnection()], pluginMenu="Home/Connections", accessibilityName="Copy JBDC String to Clipboard")
 @ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
 def copyJDBCConnectionString(conn):
-    if conn.parameterValues.has_key("schema"):
+    if "schema" in conn.parameterValues:
         params = "/"+conn.parameterValues["schema"]
     else:
         params = "/"
@@ -370,7 +369,7 @@ def newConnectionFromClipboard():
 
     parse_errors = False
     found_instances = 0
-    for line in text.encode("utf8").split("\n"):
+    for line in text.split("\n"):
         conn = connectionFromString(line)
         if not conn and not parse_errors:
             mforms.Utilities.show_error("Add Connection(s) from Clipboard", "Could not parse connection parameters from string '%s'" % line, "OK", "", "")
@@ -470,9 +469,11 @@ def startCommandLineClientForConnection(conn):
                 + "' &"   # <--- launch GUI terminal and exit (returns to Workbench immediately rather than blocking) 
         ], shell=False, env=my_env)
 
-if sys.platform == "linux2":
-    @ModuleInfo.export(grt.INT)
-    def startODBCAdmin():
+
+@ModuleInfo.plugin("wb.tools.cmdlineClient", caption="Start Command Line Client", input=[wbinputs.selectedConnection()], pluginMenu="Home/Connections", accessibilityName="Command Line Client")
+@ModuleInfo.export(grt.INT)
+def startODBCAdmin():
+    if sys.platform == "linux2":
         path = get_exe_path('iodbcadm-gtk')
         if not path:
             path = get_exe_path('ODBCManageDataSourcesQ4')
@@ -482,16 +483,12 @@ if sys.platform == "linux2":
             return 1
         else:
             return 0
-elif sys.platform == "darwin":
-    @ModuleInfo.export(grt.INT)
-    def startODBCAdmin():
+    elif sys.platform == "darwin":
         ret = subprocess.call("open -a 'ODBC Administrator'", shell=True)
         if ret == 1:
             ret = subprocess.call("open -a 'ODBC Manager'", shell=True)
         return 0 if ret == 1 else 1
-elif sys.platform == "win32":
-    @ModuleInfo.export(grt.INT)
-    def startODBCAdmin():
+    elif sys.platform == "win32":
         # WTF alert:
         # In 64bit windows, there are 2 versions of odbcad32.exe. One is 64bits and the other 32.
         # The 64bit version is in \Windows\System32
@@ -499,7 +496,7 @@ elif sys.platform == "win32":
 
         # so if we're a 64bit WB, then we run the 64bit odbc tool (since we can't use 32bit drivers anyway)
 
-        if sys.maxint > 2**31:
+        if sys.maxsize > 2**31:
             subprocess.Popen(r"%SYSTEMROOT%\SysWOW64\odbcad32.exe", shell=True, creationflags=subprocess.CREATE_NEW_PROCESS_GROUP, close_fds=True)
         else:
             subprocess.Popen(r"%SYSTEMROOT%\System32\odbcad32.exe", shell=True, creationflags=subprocess.CREATE_NEW_PROCESS_GROUP, close_fds=True)
@@ -565,7 +562,7 @@ def startUtilitiesShell():
                 f.write('rm -f "%s"\n' % setup_script)
                 f.write('sh -i\n')
                 f.close()
-                os.chmod(setup_script, 0700)
+                os.chmod(setup_script, 0o700)
 
                 if 'konsole' in term:
                     subprocess.call([term, "-e", "/bin/sh", setup_script])
@@ -591,7 +588,7 @@ class CheckForUpdateThread(threading.Thread):
         
         self.is_running = True
         try:
-            import urllib2
+            import urllib.request, urllib.error, urllib.parse
             import json
             import base64
 
@@ -628,7 +625,7 @@ class CheckForUpdateThread(threading.Thread):
                 def cancel_clicked(self):
                     self.close()
 
-            class ProxyAuthenticationHandler(urllib2.AbstractBasicAuthHandler, urllib2.BaseHandler):
+            class ProxyAuthenticationHandler(urllib.request.AbstractBasicAuthHandler, urllib.request.BaseHandler):
 
                 auth_header = 'Proxy-authorization'
                 attempts = 0
@@ -709,17 +706,17 @@ class CheckForUpdateThread(threading.Thread):
 
                     return self.parent.open(req, timeout=req.timeout)
 
-            proxy_handler = urllib2.ProxyHandler()
+            proxy_handler = urllib.request.ProxyHandler()
             proxy_auth_handler = ProxyAuthenticationHandler()
 
-            opener = urllib2.build_opener()
+            opener = urllib.request.build_opener()
             opener.add_handler(proxy_handler)
             opener.add_handler(proxy_auth_handler)
 
-            urllib2.install_opener(opener)
+            urllib.request.install_opener(opener)
 
-            self.json = json.load(urllib2.urlopen("http://workbench.mysql.com/current-release")) 
-        except Exception, error:
+            self.json = json.load(urllib.request.urlopen("http://workbench.mysql.com/current-release")) 
+        except Exception as error:
 
             self.json = None
             self.error = "%s\n\nPlease verify that your internet connection is available." % str(error)        
@@ -743,7 +740,7 @@ class CheckForUpdateThread(threading.Thread):
                 else:
                     mforms.Utilities.show_message('MySQL Workbench is Up to Date', 'You are already using the latest version of MySQL Workbench.', 'OK', '', '')
         
-            except Exception, error:
+            except Exception as error:
                 mforms.Utilities.show_error("Check for updates failed", str(error), "OK", "", "")
 
         mforms.App.get().set_status_text('Ready.')
@@ -785,7 +782,7 @@ class SSLWizard_GenerationTask:
         try:
             if not os.path.exists(self.main.certificates_root) or not os.path.isdir(self.main.certificates_root):
                 log_info("Creating certificates toor directory[%s]" % self.main.certificates_root)
-                os.mkdir(self.main.certificates_root, 0700)
+                os.mkdir(self.main.certificates_root, 0o700)
             
             if os.path.exists(self.path) and not os.path.isdir(self.path):
                 self.display_error("Checking requirements", "The selected path is a file. You should select a directory.")
@@ -794,10 +791,10 @@ class SSLWizard_GenerationTask:
                 if mforms.Utilities.show_message("Create directory", "The directory you selected does not exists. Do you want to create it?", "Create", "Cancel", "") == mforms.ResultCancel:
                     self.display_error("Create directory", "The operation was canceled.")
                     return False
-                os.mkdir(self.path, 0700)
+                os.mkdir(self.path, 0o700)
                 
             return True
-        except OSError, e:
+        except OSError as e:
             self.display_error("Create directory", "There was an error (%d) - %s\n%s" % (e.errno, str(e), str(traceback.format_exc())))
             if e.errno == 17:
                 return True
@@ -824,10 +821,10 @@ class SSLWizard_GenerationTask:
                 return False
 
             return True
-        except ValueError, e:
+        except ValueError as e:
             log_error("Running command: %s\nValueError exception\n" % (str(e.cmd)))
             return False
-        except OSError, e:
+        except OSError as e:
             log_error("Running command: %s\nException:\n%s\n" % (str(command), str(e)))
             return False
 
@@ -989,7 +986,7 @@ class SSLWizard_OptionsPage(WizardPage):
             try:
                 if os.path.isfile(filepath):
                     os.unlink(filepath)
-            except Exception, e:
+            except Exception as e:
                 log_error("SSL Wizard: Unable to remove file %s\n%s" % (filepath, str(e)))
                 return
                 
@@ -1238,7 +1235,7 @@ def generateCertificates(parent, conn, conn_id):
         log_info("Running SSL Wizard\n%s\n" % str(p))
         r = SSLWizard(p, conn, conn_id)
         r.run(True)
-    except Exception, e:
+    except Exception as e:
         log_error("There was an exception running SSL Wizard.\n%s\n\n%s" % (str(e), traceback.format_exc()))
 
 
